import { SpellingDictionary, SuggestionResult } from './SpellingDictionary';
import { genSequence } from 'gensequence';

export class SpellingDictionaryCollection implements SpellingDictionary {
    constructor(readonly dictionaries: SpellingDictionary[], readonly name: string) {
        this.dictionaries = this.dictionaries.filter(a => !!a.size);
    }

    public has(word: string) {
        return isWordInAnyDictionary(this.dictionaries, word);
    }

    public suggest(word: string, numSuggestions: number) {
        return makeSuggestions(this.dictionaries, word, numSuggestions);
    }

    public get size() {
        return this.dictionaries.reduce((a, b) => a + b.size, 0);
    }
}

export function createCollection(dictionaries: SpellingDictionary[], name: string) {
    return new SpellingDictionaryCollection(dictionaries, name);
}

export function isWordInAnyDictionary(dicts: SpellingDictionary[], word: string) {
    return !!genSequence(dicts)
        .first(dict => dict.has(word));
}

export function makeSuggestions(dicts: SpellingDictionary[], word: string, numSuggestions: number) {

    // Make a map of the unique suggestions.  If there are duplicates, keep the lowest cost.
    const allSuggestions = genSequence(dicts)
        .concatMap(dict => dict.suggest(word, numSuggestions))
        .reduceToSequence<[string, SuggestionResult], Map<string, SuggestionResult>>((map, sug) => {
            const cost = Math.min(sug.cost, (map.get(sug.word) || sug).cost);
            map.set(sug.word, { ...sug, cost });
            return map;
        }, new Map<string, SuggestionResult>())
        // Convert the [k, v] to the v
        .map(([, v]) => v)
        .toArray()
        .sort((a, b) => a.cost - b.cost);

    return allSuggestions.slice(0, numSuggestions);
}

export function createCollectionP(dicts: Promise<SpellingDictionary>[], name: string): Promise<SpellingDictionaryCollection> {
    return Promise.all(dicts)
        .then(dicts => new SpellingDictionaryCollection(dicts, name));
}
